use baseline::M2BaselineColors;
use palette::Hsl;
use serde::Serialize;

use crate::{convert::map_hsl_to_srgb_hex, errors};

use super::SchemeExport;

mod baseline;
mod color_set;
mod swatch;

#[derive(Debug, Clone, Serialize)]
pub struct MaterialDesign2Scheme {
    pub white: String,
    pub black: String,
    pub light: M2BaselineColors,
    pub dark: M2BaselineColors,
}

impl MaterialDesign2Scheme {
    pub fn new(primary: &str, secondary: &str, error: &str) -> Result<Self, errors::ColorError> {
        Ok(Self {
            white: map_hsl_to_srgb_hex(&Hsl::new(0.0, 0.0, 1.0)),
            black: map_hsl_to_srgb_hex(&Hsl::new(0.0, 0.0, 0.0)),
            light: M2BaselineColors::new(primary, secondary, error, false)?,
            dark: M2BaselineColors::new(primary, secondary, error, true)?,
        })
    }

    pub fn add_custom_color(&mut self, name: &str, color: &str) -> Result<(), errors::ColorError> {
        self.light.add_custom_color(name, color)?;
        self.dark.add_custom_color(name, color)?;
        Ok(())
    }
}

impl SchemeExport for MaterialDesign2Scheme {
    fn output_css_variables(&self) -> String {
        let mut css_variables = Vec::new();

        css_variables.push(format!("--color-white: #{};", self.white));
        css_variables.push(format!("--color-black: #{};", self.black));

        css_variables.extend(self.light.to_css_variable());
        css_variables.extend(self.dark.to_css_variable());

        css_variables.join("\n")
    }

    fn output_scss_variables(&self) -> String {
        let mut scss_variables = Vec::new();

        scss_variables.push(format!("$color-white: #{};", self.white));
        scss_variables.push(format!("$color-black: #{};", self.black));

        scss_variables.extend(self.light.to_scss_variable());
        scss_variables.extend(self.dark.to_scss_variable());

        scss_variables.join("\n")
    }

    fn output_javascript_object(&self) -> String {
        let mut js_object = Vec::new();

        js_object.push("{".to_string());

        js_object.push(format!("  white: '#{}'", self.white));
        js_object.push(format!("  black: '#{}'", self.black));

        js_object.push("  light: {".to_string());
        js_object.extend(
            self.light
                .to_javascript_object()
                .into_iter()
                .map(|s| format!("    {}", s))
                .collect::<Vec<String>>(),
        );
        js_object.push("  }".to_string());

        js_object.push("  dark: {".to_string());
        js_object.extend(
            self.dark
                .to_javascript_object()
                .into_iter()
                .map(|s| format!("    {}", s))
                .collect::<Vec<String>>(),
        );
        js_object.push("  }".to_string());

        js_object.push("}".to_string());

        js_object.join("\n")
    }
}
