import { capitalize, isEmpty } from 'lodash-es';
import { useCallback, useMemo } from 'react';
import { useColorFunction } from '../../ColorFunctionContext';
import { useCopyColor } from '../../hooks/useCopyColor';
import { ColorDescription } from '../../models';
import styles from './ColorCard.module.css';

type ColorCardProps = {
  color: ColorDescription;
  copyMode?: 'hex' | 'rgb' | 'hsl' | 'lab' | 'oklch';
};

export function ColorCard({ color, copyMode }: ColorCardProps) {
  const { colorFn } = useColorFunction();
  const copytToClipboard = useCopyColor();
  const colorHex = useMemo(() => {
    const [r, g, b] = color.rgb;
    if (colorFn) {
      try {
        const hex = colorFn.rgb_to_hex(r, g, b);
        return hex;
      } catch (e) {
        console.error('[Convert RGB]', e);
      }
    }
    return `${((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1)}`;
  }, [colorFn, color]);
  const handleCopy = useCallback(() => {
    switch (copyMode) {
      case 'rgb':
        copytToClipboard(`rgb(${color.rgb[0]}, ${color.rgb[1]}, ${color.rgb[2]})`);
        break;
      case 'hsl':
        copytToClipboard(
          `hsl(${color.hsl[0].toFixed(1)}, ${(color.hsl[1] * 100).toFixed(2)}%, ${(
            color.hsl[2] * 100
          ).toFixed(2)}%)`,
        );
        break;
      case 'lab':
        copytToClipboard(
          `lab(${color.lab[0].toFixed(1)}, ${color.lab[1].toFixed(2)}, ${color.lab[2].toFixed(2)})`,
        );
        break;
      case 'oklch':
        copytToClipboard(
          `oklch(${(color.oklch[0] * 100).toFixed(2)}%, ${color.oklch[1].toFixed(
            4,
          )}, ${color.oklch[2].toFixed(1)})`,
        );
        break;
      case 'hex':
      default:
        copytToClipboard(`#${colorHex}`);
        break;
    }
  }, [copytToClipboard, color, copyMode, colorHex]);

  return (
    <div className={styles.card} onClick={handleCopy}>
      <div
        className={styles.color_block}
        style={{ backgroundColor: `rgb(${color.rgb[0]}, ${color.rgb[1]}, ${color.rgb[2]})` }}
      />
      <div className={styles.description_line}>
        <div className={styles.title}>
          <span className={styles.name}>{color.name}</span>
          {!isEmpty(color.pinyin) && (
            <span className={styles.en_name}>{color.pinyin.map(capitalize).join(' ')}</span>
          )}
        </div>
        <div className={styles.color_value}>#{colorHex}</div>
      </div>
    </div>
  );
}
